@@ -114,8 +114,9 @@ Rcpp::List PF_smooth(
     arma::mat &Q, const arma::mat Q_tilde, const Rcpp::List &risk_obj,
     const arma::mat &F, const int n_max, const int n_threads,
     const arma::vec &fixed_parems, const int N_fw_n_bw, const int N_smooth,
+    const int N_smooth_final,
     Rcpp::Nullable<Rcpp::NumericVector> forward_backward_ESS_threshold,
-    const int debug, const int N_first, std::string type,
+    const int debug, const int N_first, std::string type, const int nu,
 
     /* non-common arguments */
     const std::string method, const std::string smoother, const std::string model){
@@ -127,13 +128,15 @@ Rcpp::List PF_smooth(
         n_fixed_terms_in_state_vec,
         X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
         risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, forward_backward_ESS_threshold, debug, N_first));
+        N_smooth, N_smooth_final, forward_backward_ESS_threshold, debug,
+        N_first, nu));
   else if (type == "VAR")
     data.reset(new PF_data(
         n_fixed_terms_in_state_vec,
         X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
         risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, forward_backward_ESS_threshold, debug, N_first));
+        N_smooth, N_smooth_final, forward_backward_ESS_threshold, debug,
+        N_first, nu));
   else
     Rcpp::stop("'type' not implemented");
 
@@ -245,25 +248,27 @@ Rcpp::List particle_filter(
     const arma::mat &F, const int n_threads,
     const arma::vec &fixed_parems, const int N_fw_n_bw,
     Rcpp::Nullable<Rcpp::NumericVector> forward_backward_ESS_threshold,
-    const int debug, const int N_first, std::string type,
+    const int debug, const int N_first, const int nu, std::string type,
     const bool is_forward, const std::string method, const std::string model){
   const arma::ivec is_event_in_bin =
     Rcpp::as<arma::ivec>(risk_obj["is_event_in"]);
 
   std::unique_ptr<PF_data> data;
-  const unsigned int N_smooth = 1, n_max = 1;
+  const unsigned int N_smooth = 1, n_max = 1, N_smooth_final = 1;
   if(type == "RW")
     data.reset(new random_walk<PF_data>(
         n_fixed_terms_in_state_vec,
         X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
         risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, forward_backward_ESS_threshold, debug, N_first));
+        N_smooth, N_smooth_final,
+        forward_backward_ESS_threshold, debug, N_first, nu));
   else if (type == "VAR")
     data.reset(new PF_data(
         n_fixed_terms_in_state_vec,
         X, fixed_terms, tstart, tstop, is_event_in_bin, a_0, R, R.t(), Q_0, Q,
         risk_obj, F, n_max, n_threads, fixed_parems, Q_tilde, N_fw_n_bw,
-        N_smooth, forward_backward_ESS_threshold, debug, N_first));
+        N_smooth, N_smooth_final,
+        forward_backward_ESS_threshold, debug, N_first, nu));
   else
     Rcpp::stop("'type' not implemented");
 
